from system import logger
from scipy import atleast_2d, take, ravel
import Numeric
import copy

# list shared between datasets of already used dimension names
_all_dims = []

def make_dim_name_unique(suggestion):
        postfix, num = "", 0 
        while suggestion+postfix in _all_dims:
            num += 1
            postfix = "_"+str(num)                    
        return suggestion + postfix

# helper method for rpy/gtk init-bug. will convert a number to scientific format, 
# which isn't affected by the bug.        
def make_number_scientific(num):
    num = str(num)
    if not "." in num:
        return num    
    trail = len(num) - num.index(".") - 1
    return num.replace(".","") + "e-"+str(trail)

# makes a list unique (ie: supplied list is altered!)    
def make_list_unique(list, to_add):
    if list:
        for x,i in enumerate(list):
            while list.count(list[x])>1:
                list[x] = list[x] + to_add           
    return list
    
class IdentifierMismatchError(Exception):
    """Identifiers provided don't match array shape."""
    pass

class Dataset:
    """The Dataset base class.
    
    A Dataset is an n-way array with defined string identifiers across
    all dimensions.

    example of use:

    ---
    dim_name_rows = 'rows'
    names_rows = ('row_a', 'row_b')
    ids_1 = [dim_name_rows, names_rows]

    dim_name_cols = 'cols'
    names_cols = ('col_a', 'col_b', 'col_c', 'col_d')
    ids_2 = [dim_name_cols, names_cols]

    Array_X = rand(2, 4)
    data = Dataset(Array_X, (ids_1, ids_2), name="Testing")

    dim_names = [dim for dim in data]

    column_identifiers = [ident for ident in data['cols'].keys()]
    column_index = [index for index in data['cols'].values()]

    'cols' in data -> True

    ---

    data = Dataset(rand(10, 20)) (generates dims and ids (no links))
    """
    
    def __init__(self, array, identifiers=None, name="Unnamed data"):    
        # 'None' values in identifiers means they will be chosen automagically
        array = Numeric.asarray(atleast_2d(array))
                
        # if only identifier, assume dataset was 1d and has been converted to 2D
        if identifiers and len(identifiers)==1:
            identifiers = [(None, None)] + identifiers

        if not identifiers:
            identifiers = [(None, None)]* len(array.shape)
        
        assert len(identifiers) == len(array.shape) # provided identifiers must be same length
        
        self._name = name
        self._dims = [] #existing dimensions in this dataset
        self._map = {} # internal mapping for dataset:  identifier <--> index
        
        self.shape = array.shape

        self._set_identifiers(identifiers)
        for dim, dim_length in zip(self.get_dim_names(), self.shape):
            num_idents = len(self.get_identifiers(dim))
            if num_idents != dim_length:
                raise IdentifierMismatchError, "%d identifiers in %s, doesn't match shape (%d)." % \
                      (num_idents, dim, dim_length)
        
        self._array = array
        self.generate_description()
                          
    def __iter__(self):
        """Returns an iterator over dimensions of dataset."""
        return self._dims.__iter__()

    def __contains__(self, dim):
        """Returns True if dim is a dimension name in dataset."""
        return self._map.__contains__(dim)

    def __len__(self):
        """Returns the number of dimensions in the dataset"""
        return len(self._map)

    def __getitem__(self, dim):
        """ Converts between dim/id names and positions"""
        id = None
        if isinstance(dim, int):
            return self.get_dim_names()[dim]
        if isinstance(dim, str):
            return self.get_dim_names().index(dim)
        if isinstance(dim, tuple):
            dim, id = dim
            if isinstance(dim, int):
                dim = self.get_dim_names()[dim]           
            if isinstance(id, int):
                return self.get_identifiers(dim)[id]
        return self._map[dim][id]
        
    def _create_identifiers(self, dim_number, number):
        """ Creates a new set of identifiers """
        return ["%s" % (i,) for i in range(1,number+1)]
         
    def _suggest_dim_name(self, axis_number):
        """Suggest a unique name for axis and return it"""
        presets = {0 : ' Rows', 1 : ' Cols'}
        
        if axis_number in presets:
            dim_name = self.get_name() + presets[axis_number]
        else:
            dim_name = self.get_name() + " Axis " + str(axis_number)
        
        return dim_name


    def _set_identifiers(self, identifiers):
        for axis, (dim, ids) in enumerate(identifiers):
            pos_map = {}

            # no dim given? generate one
            if not dim:
                dim = self._suggest_dim_name(axis)
                
            # storing all dims to ensure uniqueness
            if dim not in _all_dims:
                _all_dims.append(dim)
            if dim not in self._dims:
                self._dims.append(dim)
            else:
                raise ValueError, "Dimension names must be unique"

            # no ids? generate'em!
            if not ids:
                ids = self._create_identifiers(self._dims.index(dim), self.shape[axis])

            # generate map
            for index, ident in enumerate(ids):
                    pos_map[ident] = index
                
            self._map[dim] = pos_map
        
    def asarray(self):
        """Returns the numeric array (data) of dataset"""
        return self._array

    def get_name(self):
        """Returns dataset name"""
        return self._name

    def set_name(self, name):
        """ Sets dataset name. Updates first line of description"""
        self._name = name
        self.description[0] = "<span weight=\"bold\">Data:</span> %s" % self.get_name()

    def get_description(self):
        """Get description (used for information pane)"""
        return "\n".join(self.description)
        
    def add_to_description(self, text):
        """ Adds text to description"""
        self.description.append(str(text))
                        
    def generate_description(self):
        text = zip(self.get_dim_names(), self.shape)
        text = ", ".join(["%s (%d)" % dim for dim in text])                             
        self.description = ["<span weight=\"bold\">Data:</span> %s" % self.get_name(),
"<span weight=\"bold\">Dimensions:</span> %s" % text]                        

    def get_all_dims(self):
        """Returns all dimensions created during current session"""
        return _all_dims

    def get_dim_names(self):
        """Returns dim names""" 
        return [dim for dim in self._dims]
        
    def change_dim_name(self,old,new):
        if new in self._dims or old not in self._dims:
            return False
        index = self._dims.index(old)
        self._dims.remove(old)
        self._dims.insert(index,new)
        self._map[new] = self._map[old]
        del self._map[old]
        
        self.generate_description()    
        return True                
                
    def change_identifiers(self, dim, identifiers):
        assert len(identifiers)==len(self._map[dim])
        self._map[dim]={}
        for val, key in enumerate(identifiers):
            self._map[dim][str(key)] = val

    def get_identifiers(self, dim, indices=None):
        """Returns identifiers along dim, sorted by position (index).

        You can optionally provide a list of indices to get only the
        identifiers of a given position.
        """
        items = self._map[dim].items()
        backitems = [[v[1], v[0]] for v in items]
        backitems.sort()
        sorted_ids = [backitems[i][1] for i in range(0, len(backitems))]

        # we use id as scipy-arrays return a new array on boolean
        # operations
        if id(indices) != id(None):
            return [sorted_ids[index] for index in indices]
        else:
            return sorted_ids

    def get_indices(self, dim, idents):
        """Get indices for identifiers along dimension."""
        return [self._map[dim][key] for key in idents]
    
    def get_identifiers_from_indices(self, dim, indices):
        """Get identifiers for identifiers along dimension"""
        ids = self.get_identifiers(dim)
        return [ids[pos] for pos in indices]
        
    def get_data_slice(self, dim, ident):
        """Get slice of original matrix"""
        axis = self.get_dim_names().index(dim) 
        item = self.get_identifiers(dim).index(ident)
        return ravel(take(self._array ,[item] , axis=axis))

    def get_dimension_length(self,dim):
        """Get number of variables in dimension (this does not equal number of 
        ids returned when getting a slice!)"""
        return len(self._map[dim])

    def remove_objects(self, dim, idents):
        """Create new Dataset with identifiers along dimension removed.

        Needs to be overridden in subclasses if they're not based on
        arrays.
        """
        
        # only delete objects which actually exist
        idents = set(self.get_identifiers(dim)).intersection(idents)
        
        dim_index = self.get_dim_names().index(dim)

        # we invert list of indices to get the remaining objects
        r = range(self.shape[dim_index])
        for i in self.get_indices(dim, idents):
            r.remove(i)

        # then we get the identifiers for remaining objects
        all_identifiers = []
        for i, dim_name in enumerate(self.get_dim_names()):
            identifiers = self.get_identifiers(dim_name)
            if i == dim_index:
                identifiers = map(identifiers.__getitem__, r)
            all_identifiers.append((dim_name, identifiers))

        # and we create a new array with only remaining objects
        a = take(self.asarray(), r, dim_index)
        new_dataset = Dataset(a, all_identifiers, name=self.get_name())
        new_dataset.add_to_description("<b>Generated from</b>: " + self.get_name())
        new_dataset.add_to_description("<b>Removed from %s</b>: %s" % (dim, ", ".join(idents)))
        return new_dataset

    def get_selected_objects(self, selection):
        """Get number of objects selected in each dimension given selection.

        Returns a dictionary indexed by name of dimension. Only
        dimensions with an actual selection will be returned.
        """
        result = {}
        for dim_name, idents in selection.items():
            if dim_name in self.get_dim_names():
                count = len(set(self.get_identifiers(dim_name)).intersection(idents))
                result[dim_name] = count
        return result
        
class RDataset(Dataset):
    """ Dataset for representing complix data.frames in R.
        Advanced objects of type robj in python (methods f.x) will not be dropped.
        check get_ignored_names() to see if anything was dropped.
        Use insert_onto_R for restoring complex objects. 
       """
    def __init__(self,robject_name, default_object,name="New Dataset"):        
        import rpy

        self._r_names = []
        self._r_arrays = []
        self._r_ignored_names = []
        
        silent_eval = rpy.with_mode(rpy.NO_CONVERSION, rpy.r)
        robjType = type(silent_eval("1"))
        
        # saving vars for later use
        for varname in rpy.r("names("+robject_name+")"):
            fetchstr = robject_name + "$" +  varname
            data = rpy.r(fetchstr)                        
            if  type(data) == robjType:
                self._r_ignored_names.append(varname)
            else:
                self._r_names.append(varname)
                self._r_arrays.append(data)
        
        # creating matrix which will be shown as default                
        if not default_object in self._r_names:
            default_object = self._r_names[0]                                                              
        main_array = self._r_arrays[self._r_names.index(default_object)]
        
        if type(main_array) == dict:
            ids = list(set(rpy.r("names("+robject_name+"$"+default_object+")")))
            values = [main_array[key] for key in ids]               
            Dataset.__init__(self,Numeric.reshape(values,(len(values),1)),[(None,ids),(None,None)],name=name)
        else:
            Dataset.__init__(self,main_array,name=name)                        
                

    """ Inserts data.frame into R. Will overwrite. """ 
    def insert_into_R(self,name,set_class = None):
        import rpy
        name = str(name)
        rpy.r("c() -> " + name)
        self.insert_onto_R(name,set_class = set_class)  

    """ Writes data onto a data.frame.
        Running a function and then overwriting it with the old values is quick 
        and easy way of getting back the old functional object """          
    def insert_onto_R(self,name, set_class = None):
        import rpy            
        for x, ii in enumerate(self._r_names):
            rpy.r.assign("temp", self._r_arrays[x])
            rpy.r(name+"$"+ii+" <- temp")
        if set_class:
            rpy.r('class('+name+') <- "' + str(set_class)+'"')

    def get_ignored_names(self):
        return self._r_ignored_names
        
def get_dataset_from_r_matrix(name,**kwargs):
    import rpy    
    data = rpy.r(name)    
    
    colnames = make_list_unique(rpy.r("colnames("+name+")"),"_")
    rownames = make_list_unique(rpy.r("rownames("+name+")"),"_")                   
    return Dataset(data,[(None, rownames),(None,colnames)],name=name)            

