import gobject
import gtk
import time
import config 

class Logger:
    def __init__(self):
        self.store = gtk.ListStore(gobject.TYPE_STRING, 
                                   gobject.TYPE_STRING, 
                                   gobject.TYPE_STRING)
        self.levels = ['debug', 'notice', 'warning', 'error']
        self.level_text = {'debug': 'Debug', 
                           'notice': 'Notice', 
                           'warning': 'Warning', 
                           'error': 'Error'}
        self.components = {}
        self.colors = { 'debug': 'grey',
                        'notice': 'black',
                        'warning': 'brown',
                        'error': 'red' }

    def log(self, level, message):
        iter = self.store.append()
        self.store.set_value(iter, 0, level)
        self.store.set_value(iter, 1, message)
        self.store.set_value(iter, 2, self.colors[level])

    def level_number(self, level):
        return self.levels.index(level)

class LogView(gtk.TreeView):

    def __init__(self, logger=None, level='notice'):
        self.logger = logger 
        self.model = logger.store
        self.level = config.logg_level
        self.level_no = logger.level_number(level)

        # Set up filter
        self.filter = self.model.filter_new()
        gtk.TreeView.__init__(self, self.filter)
        self.filter.set_visible_func(self.level_filter)
        self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

        # Set up log level column
        renderer = gtk.CellRendererText()
        self.level_col = gtk.TreeViewColumn('Level', renderer, text=0)
        self.level_col.add_attribute(renderer, "foreground", 2)
        self.append_column(self.level_col)

        # Set up message column
        renderer = gtk.CellRendererText()
        self.message_col = gtk.TreeViewColumn('Message', renderer, text=1)
        self.message_col.add_attribute(renderer, "foreground", 2)
        self.append_column(self.message_col)

        # Activate context menu
        self.menu = LogMenu(self.logger, self)
        self.connect('popup_menu', self.popup_menu)
        self.connect('button_press_event', self.mouse_popup_menu)

        # Make sure tree view displays bottom entry when entered
        def scroll_to_last(model, path, it):
            scroll_level = self.logger.level_number(model.get_value(it, 0))
            actual_level = self.logger.level_number(config.logg_level)
            if path and scroll_level >= actual_level:
                self.scroll_to_cell(path)
        self.model.connect('row-changed', scroll_to_last)

    def set_level(self, level):
        self.level = level
        self.level_no = self.logger.levels.index(level)
        self.filter.refilter()
        self.queue_draw()
        config.logg_level = level
        config.write_config()

    def popup_menu(self, *rest):
        self.menu.popup(None, None, None, 0, 0)

    def mouse_popup_menu(self, widget, event):
        if event.button == 3:
            self.menu.popup(None, None, None, event.button, event.time)

    def level_filter(self, store, iter):
        if store.get_value(iter, 0):
            value = self.logger.levels.index(store.get_value(iter, 0))
            return value >= self.level_no
        else:
            return False


class LogLevelMenu(gtk.Menu):
    def __init__(self, logger, view):
        self.logger = logger
        self.view = view
        items = []
        gtk.Menu.__init__(self)

        for level in logger.levels:
            if len(items) == 0:
                group = None
            else:
                group = items[0]
            item = gtk.RadioMenuItem(group, logger.level_text[level], level)
            item.connect('activate', self.set_log_level, level)
            items.append(item)
            self.append(item)
            item.show()

        item_no = logger.level_number(view.level)
        items[item_no].set_active(True)

    def set_log_level(self, widget, level, *rest):
        if widget.active:
            self.view.set_level(level)

class LogComponentMenu(gtk.Menu):
    def __init__(self, logger, view):
        gtk.Menu.__init__(self)
        components = logger.components.keys()
        components.sort(str.__gt__)
        
        for c in components:
            item = gtk.MenuItem(c)
            self.append(item)
            item.show()

#        for component in logger.components
class LogMenu(gtk.Menu):
    def __init__(self, logger, view):
        gtk.Menu.__init__(self)
        self.logger = logger
        self.view = view

        # View Log Level
        self.view_menu = LogLevelMenu(logger, view)
        self.view_item = gtk.MenuItem('View Log Level')
        self.view_item.set_submenu(self.view_menu)
        self.append(self.view_item)
        self.view_item.show()

        # View Components
        self.component_menu = LogComponentMenu(logger, view)
        self.component_item = gtk.MenuItem('View Components')
        self.component_item.set_submenu(self.component_menu)
        self.append(self.component_item)
        self.component_item.show()

        # Copy log message
        copy_item = gtk.MenuItem('Copy')
        copy_item.connect('activate', self.activate_copy_button)
        self.append(copy_item)
        copy_item.show()
        
        # Clear Log
        clear_item = gtk.MenuItem('Clear Log')
        clear_item.connect('activate', self.activate_clear_button)
        self.append(clear_item)
        clear_item.show()
        self.cb = gtk.Clipboard()

    def activate_clear_button(self, item):
        self.logger.store.clear()

    # copy selected log message to clipboard
    def activate_copy_button(self, item):
        model, paths = self.view.get_selection().get_selected_rows()
        message = ""
        for p in paths:
            iter = model.get_iter(p)
            message += model.get_value(iter, 0) + "\t"
            message += model.get_value(iter, 1) + "\n"
            
        message = message[:-1]
        self.cb.set_text(text=message, len=-1)
         

class InformationPane(gtk.Label):      
    def __init__(self):
        gtk.Label.__init__(self)
        self.set_alignment(0,0)
        self.set_selectable(True)

    def inform_about_selected(self, dim, ids):
        # generation of belong in a selection class.. there isn't one yet however...
        ids = ", ".join(ids)
        ids = [ids[x:x+100] for x in xrange(0,100*6,100)]
        if len(ids[-1])>100:
            ids[-1] = id[-1][0:80]+" ....."
                            
        text = """\
<span weight="bold">Selection:</span> %s
<span weight="bold">Identifiers:</span>\n%s""" % (dim,"\n".join(ids))  
        self.set_markup(text)
        self.get_focus()          

    def inform_about_dataset(self, dataset):
        text = dataset.get_description()  
        self.set_markup(text)
        self.get_focus()
        
    def set_from_docstring(self, foo):
        if foo.__doc__:
            self.set_markup(foo.__doc__)
        else:
            self.set_markup("No doc-string available.")
        self.get_focus()
        
    def get_focus(self):        
        parent = self.parent
        
        # go up and find first NoteBook instance 
        while(parent and not isinstance(parent, gtk.Notebook)):
            parent = parent.parent
        if parent:
            parent.set_current_page(0)                                    
    
logger = Logger()
log = logger.log

information = InformationPane()


