import gtk
import gobject
import time
import os
import rpy, scipy, Numeric
from gtk import gdk
from system import dataset, logger, plots, workflow, dialogs, filehandlers
from system.project import ActivatableItem, Selection, CategoryItem


import Numeric

class NavigatorView (gtk.TreeView):
    def __init__(self, gladefilename):
        gtk.TreeView.__init__(self)
        
        self._project = None
        self._gladefilename = gladefilename
        self._data_tree = None
        self._plot_view = None
        self._object_col = None
        
        if hasattr(self,"set_enable_tree_lines"):
            self.HAS_GTK210 = True
        else:
            self.HAS_GTK210 = False

        # add selection to enable context menus for more than one item
        self._selection = None
        self._paths = None                    

        self._init_gui()
        
    def _init_gui(self):
        """Set up visualization of tree model and connect events."""
        if self.HAS_GTK210:
            # if gtk210 use proper tree lines instead of "-" image
            self.set_enable_tree_lines(True)            
        
        # various treeview-related properties
        self.set_headers_visible(False)

        # selection mode
        self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self.get_selection().set_select_function(self.is_selectable)
        self.get_selection().connect('changed', self.selection_changed_handler)

        # set up text renderers for object column
        textrenderer = gtk.CellRendererText()
        textrenderer.connect("edited", self.cell_edited)                
        pixbufrenderer = gtk.CellRendererPixbuf()                                                                                                      
        object_column = gtk.TreeViewColumn('Object')
        object_column.pack_start(pixbufrenderer, expand=False)
        object_column.set_attributes(pixbufrenderer, pixbuf=5)        
        object_column.pack_start(textrenderer, expand=False)
        object_column.set_attributes(textrenderer, cell_background=2,
                                     foreground=3, text=0, editable=4)
                                     
        self.append_column(object_column)
        self._object_col = object_column

        # send events to plots / itself
        self.enable_model_drag_source(gdk.BUTTON1_MASK,
                                      [("GTK_TREE_MODEL_ROW",
                                        gtk.TARGET_SAME_APP, 7)],
                                      gdk.ACTION_LINK | gdk.ACTION_DEFAULT)
        self.connect("drag-data-get", self.slot_drag_data)
        self.connect("button-press-event", self.slot_button_pressed)
        self.connect("key_release_event", self.slot_key_released)
        self.connect('row_activated', self.row_activated_handler)

        # right click context menu
        self.connect("button-release-event", self.button_released_event)                                        


        
    def slot_drag_data(self, treeview, context, selection, target_id, etime):
        """Sets data for drag event."""
                    
        treeselection = treeview.get_selection()
        model, paths = treeselection.get_selected_rows()
        if paths:    
            self._data_tree.drag_data_get(paths[0], selection)

    def add_project(self, project):
        """Sets treestore."""
        self._project = project
        self._data_tree = project.data_tree
        self._data_tree.set_use_function_icon(not self.HAS_GTK210)
        self.set_model(project.data_tree)
        self._data_tree.connect('row-changed', self.row_changed_handler)
        self.expand_all()
           
    def set_plot_main_view(self, plot_main_view):
        """Set reference to the plot main view."""
        self._plot_view = plot_main_view
   
    def is_selectable(self, path):
        if self._data_tree:
            obj = self._data_tree.get_object_at_path(path)
            if not obj:
                return False
            if not isinstance(obj, dataset.Dataset):
                return False
        return True

    def selection_changed_handler(self, selection):
        """Handle changes to selection when user clicks datasets."""
        # To keep order of selection we need to adjust the existing
        # list of selected datasets instead of just using the one
        # passed
        model, paths = selection.get_selected_rows()
        selected_data = [self._data_tree.get_object_at_path(path) for path in paths] 
        last_selection = self._project.get_current_data()
        # we only keep still-selected datasets from the previous
        # selection ...
        current_selection = [data for data in last_selection \
                             if data in selected_data]
                              
        # ... then we add the newly selected datasets that we didn't
        # already have selected
        current_selection += [data for data in selected_data \
                              if data not in current_selection]

        # update information-pane             
        if current_selection:                                                   
            logger.information.inform_about_dataset(current_selection[-1])                                                                 
                              
        self._project.set_current_data(current_selection)                                                         

        # store selection to enable context menu for multiple items
        self._selection = current_selection # multiple context
        self._paths = paths
            
    def row_changed_handler(self, treestore, path, iter):
        self.expand_to_path(path)        
        self.set_cursor(path)
        self.grab_focus()

    def row_activated_handler(self, widget, path, column):
        obj = self._data_tree.get_object_at_path(path)
        if isinstance(obj, plots.Plot):
            logger.log('debug', 'Activating plot')
            self._plot_view.insert_view(obj)
        if isinstance(obj, dataset.Dataset):
            logger.log('debug','Inserting Table')
            self._plot_view.insert_view(plots.get_table(obj)[0])                        
        elif isinstance(obj, ActivatableItem):
            obj.activate()
        else:
            pass
            # does it really matter?
            
            #t = type(obj)
            #logger.log('warning',
            #           'Activated datatype was %s. Don\'t know what to do.' % t)

    def button_released_event(self, list, event):
        """Show context menu on rightclick."""
        def new_window_activated(menuitem, target):
            self._plot_view.new_plot_window(target)

        def delete_activated(menuitem, path):
            self.delete_items([path])

        def rename_activated(menuitem, path):
            self._data_tree.set_renamable_by_path(path)
            self.grab_focus()
            self.set_cursor(path, focus_column=self._object_col,
                            start_editing=True)
                            
        def plot_density(menuitem, target):
            self._project.add_data([target],plots.get_density_plot(target),fun=None)

        def plot_mesh(menuitem, target):
            from system.plot3d import glworkflow
            self._project.add_data([target], glworkflow.MeshPlotFunction().run(target),fun=None)                            

        def delete_objects_from_dataset(menuitem, dataset, dim_name, idents):
            new_dataset = dataset.remove_objects(dim_name, idents)
            self._project.add_data([dataset], [new_dataset],
                                   fun=None)
        def plot_hist(menuitem, target):
            self._project.add_data([target], plots.get_histogram_plot(target),fun=None)
                                   
        def plot_bar(menuitem, target):
            self._project.add_data([target], plots.get_bar_plot(target),fun=None)
        
        def plot_lines(menuitem, target):
            self._project.add_data([target], plots.get_multiple_lines_plot(target),fun=None)
                                   
        def plot_line(menuitem, target):
            self._project.add_data([target], plots.get_line_plot(target),fun=None)
        
        def plot_scatter(menuitem, target):
            self._project.add_data([target], plots.get_scatter_plot(target),fun=None)

        def plot_image(menuitem, target):
            self._project.add_data([target], plots.get_imageplot(target),fun=None)                                               

        def export_excel(menuitem, target):
            filehandlers.ExcelWriter().run(target)
            
        def export_text(menuitem, target):
            filehandlers.ExportTableFunction().run(target)
        
        def export_matlab(menuitem, target):
            filehandlers.SaveMatlabFunction().run(target)
            
        def export_r(menuitem, target):
            filehandlers.RSaveFunction().run(target)
        
        def get_table(menuitem, target, only_selected, inverted):
            self._project.add_data([target], plots.get_table(target,only_selected, inverted),fun=None)

        def get_transposed(menuitem, target):
            data = StandardOperations().transpose(target)
            self._project.add_data([target],data,fun=None)

        def get_msc(menuitem, target):
            data = StandardOperations().msc(target)
            self._project.add_data([target],data,fun=None)        

        def get_autoscaled(menuitem, target):
            data = StandardOperations().autoscale(target)
            self._project.add_data([target],data,fun=None)

        def get_centered(menuitem, target):
            data = StandardOperations().center(target)
            self._project.add_data([target],data,fun=None)                    

        def get_subset(menuitem, target):
            data = StandardOperations().subset(target)
            self._project.add_data([target],data,fun=None)
        
        def get_reduce_dim(menuitem, target):
            data = StandardOperations().reduce_dim(target)
            self._project.add_data([target],data,fun=None)  
              
        # only show menu on right click
        if event.button != 3:
            return

        class widgets:
            def __init__(self, tree):
                self.tree = tree
            def __getitem__(self,key):
                return self.tree.get_widget(key)

        # Only go on if an item was clicked
        item_clicked = self.get_path_at_pos(*map(int, event.get_coords()))
        if not item_clicked:
            return

        widgets  = widgets(gtk.glade.XML(self._gladefilename,
                                     'navigator_context_menu'))
                                     
        # now, start disabling items!
        clicked_path, column, x, y = item_clicked
        model = self.get_model()
        clicked_target = model.get_object_at_path(clicked_path)
        
        # init variables relevant for what's available
        # NOTE: we're assuming datasets can be selected and everything else not. If this changes,
        # this code will break.
        
        is_dataset = isinstance(clicked_target, dataset.Dataset)
        is_plot    = isinstance(clicked_target, plots.Plot)                
        is_matrix = False
        is_cubic  = False
        use_excel  = False
        use_3dplot = False
        
        # set variables 
        if is_dataset and Numeric.product([len(x.shape)-1 for x in self._selection])==1:
            is_matrix = True
        if is_dataset and Numeric.product([len(x.shape)-2 for x in self._selection])==1:
            is_cube = True            
        if is_dataset and is_matrix:
            has_excel = filehandlers.ensure_excel_available(show_warnings = False)             
        try:
            from gtk.gtkgl.apputils import GLScene
            from system.plot3d import glworkflow
            use_3dplot = True        
        except ImportError:        
            pass
  
        # grey out everything that won't work / doesn't make sense
        widgets['show_in_new_window1'].set_sensitive(is_plot)
                        
        widgets['plot1'].set_sensitive(is_dataset)
        widgets['table3'].set_sensitive(is_dataset)
        widgets['transforms1'].set_sensitive(is_dataset)
        widgets['export_as1'].set_sensitive(is_dataset)
        widgets['export_into1'].set_sensitive(is_dataset)
     
        widgets['rename73'].set_sensitive(len(self._selection)==1)     
     
        if is_dataset:
            # -- plots --
            widgets['mesh_plot1'].set_sensitive(use_3dplot and is_matrix)
            widgets['scatter_plot1'].set_sensitive(is_matrix)
            widgets['line_plot1'].set_sensitive(is_matrix)
            widgets['bar_plot1'].set_sensitive(is_matrix)
            widgets['histogram_plot1'].set_sensitive(is_matrix)
            widgets['density_plot1'].set_sensitive(is_matrix)
            widgets['line_plots1'].set_sensitive(is_matrix)
            widgets["image_plot1"].set_sensitive(is_matrix or is_cube)
            
            # -- tables --
            # they all "work"
            
            # -- transforms --
            widgets['transpose1'].set_sensitive(True)  # silly, done for completeness
            widgets['msc1'].set_sensitive(is_matrix) 
            widgets['autoscale1'].set_sensitive(is_matrix)
            widgets['center1'].set_sensitive(is_matrix)
            widgets['get_subset1'].set_sensitive(True)
            widgets['reduce_dim1'].set_sensitive(True)
            
            # -- exports
            widgets['microsoft_excel1'].set_sensitive(use_excel and is_matrix)
            widgets['text1'].set_sensitive(is_matrix)
            widgets['matlab1'].set_sensitive(is_matrix)
            widgets['r1'].set_sensitive(is_matrix or is_cube)
                    
        # process selection list in reverse order to prevent deletion issues
        items = zip(self._selection, self._paths)
                                
        # connecting everything (access control done by greying out anyways)                       
        for (target,path) in items[::-1]:
            if is_dataset:
                set_name = target.get_name()
                selection = self._project.get_selection()
                for dim_name, num_idents in \
                        target.get_selected_objects(selection).items():
                    if num_idents == 1:
                        text = "Delete 1 object from dataset %s, dimension %s" % (set_name, dim_name)
                    else:
                        if num_idents > 1:
                            text = "Delete %d objects from dataset %s, dimension %s" % (num_idents, set_name, dim_name)
                    if num_idents > 0: # only display menuitem if there are objects selected in set
                        menuitem = gtk.MenuItem(text, use_underline=False)
                        menuitem.connect("activate", delete_objects_from_dataset,
                                 target, dim_name, selection[dim_name])
                        menuitem.show()
                        widgets['navigator_context_menu'].add(menuitem)
             
            widgets['density_plot1'].connect("activate",plot_density,target)           
            widgets['mesh_plot1'].connect("activate", plot_mesh, target)                                                        
            widgets['scatter_plot1'].connect("activate", plot_scatter, target)
            widgets['line_plot1'].connect("activate", plot_line, target)
            widgets['line_plots1'].connect("activate", plot_lines, target)
            widgets["image_plot1"].connect("activate", plot_image, target)
            widgets['bar_plot1'].connect("activate", plot_bar, target)
            widgets['histogram_plot1'].connect("activate",plot_hist,target)
            widgets['full1'].connect("activate",get_table, target, False, False)        
            widgets['selected1'].connect("activate",get_table, target, True, False)
            widgets['not_selected1'].connect("activate",get_table, target, True, True)
            widgets['transpose1'].connect("activate",get_transposed,target)
            widgets['autoscale1'].connect("activate",get_autoscaled,target)
            widgets['center1'].connect("activate",get_centered,target)
            widgets['get_subset1'].connect("activate",get_subset,target)
            widgets['msc1'].connect("activate",get_msc,target)
            widgets['reduce_dim1'].connect("activate",get_reduce_dim,target)
              
            widgets['microsoft_excel1'].connect("activate", export_excel, target)
            widgets['text1'].connect("activate", export_text, target)
            widgets['matlab1'].connect("activate", export_matlab, target)
            widgets['r1'].connect("activate", export_r, target)
                        
            widgets["delete1"].connect("activate", delete_activated, path)
            widgets['rename73'].connect("activate" , rename_activated, path)
             
        widgets["delete1"].connect("activate", delete_activated, clicked_path)
        widgets['show_in_new_window1'].connect("activate", new_window_activated, clicked_target)             
                            
        widgets['navigator_context_menu'].popup(None, None, None, 0, 0)
    
    def slot_button_pressed(self, list, event):
        # allow only plots to be draggable
        if event.button != 1:
            return
        x, y = map(int, event.get_coords())
        path = self.get_path_at_pos(x, y)
        if not path:
            return
        obj = self._data_tree.get_object_at_path(path[0])
        if not isinstance(obj,CategoryItem): 
            self.enable_model_drag_source(gdk.BUTTON1_MASK,
                                          [("GTK_TREE_MODEL_ROW",
                                            gtk.TARGET_SAME_APP, 7)],
                                          gdk.ACTION_LINK | gdk.ACTION_DEFAULT)
        else: 
            self.unset_rows_drag_source()
    
    def slot_key_released(self, widg, event):
        # handle key inputs
        if event.keyval == gtk.keysyms.Delete:
            selection = self.get_selection()
            model, selected = selection.get_selected_rows()
            if selected:
                self.delete_items(selected)
        if event.keyval == gtk.keysyms.Escape:
            self.get_selection().unselect_all()
    
    def cell_edited(self, cellrenderertext, path, text):
        # update objects and treemodel after renaming
        self._data_tree.set_renamable_by_path(path, False)
        self._data_tree.set_text_by_path(path, text)
    
    def delete_items(self, paths):
        """Deletes one or more items from the datatree."""
        # warn if more than one node is affected
        if len(paths) > 1 or self._data_tree.path_has_children(paths[0]):
            qres = dialogs.get_yes_no("Delete items", "Do you really want to delete these items and all their subitems?")
            if not qres:
                return

        # remove plots from plot_view
        for path in paths:
            for obj in self._data_tree.get_objects_from_path(path):
                if isinstance(obj, plots.Plot):
                    self._plot_view.remove_plot(obj)
        
        self._data_tree.delete_items(paths)
  
class SelectionView(gtk.TreeView):
    def __init__(self, gladefilename):       
        gtk.TreeView.__init__(self)
        
        self._project = None
        self._gladefilename = gladefilename
        self._data_tree = None
        self._object_col = None
        
        # create initial tree-model       
        if hasattr(self,"set_enable_tree_lines"):
            self.HAS_GTK210 = True
        else:
            self.HAS_GTK210 = False
                    
        self._init_gui()      

    def add_project(self, project):
        """Sets treestore."""
        self._project = project
        self._data_tree = project.selection_tree
        #self._data_tree.set_use_function_icon(not self.HAS_GTK210)
        self.set_model(project.selection_tree)
        #self._data_tree.connect('row-changed', self.row_changed_handler)
        self.expand_all()
        
    def _init_gui(self):
        """Set up visualization of tree model and connect events."""
        
        if self.HAS_GTK210:
            # if gtk210 use proper tree lines instead of "-" image
            self.set_enable_tree_lines(True)            
        
        # various treeview-related properties
        self.set_headers_visible(True)

        # selection mode
        self.get_selection().set_mode(gtk.SELECTION_SINGLE)
        self.get_selection().connect('changed', self.selection_changed_handler)

        # set up text renderers for object column
        textrenderer = gtk.CellRendererText()               
        pixbufrenderer = gtk.CellRendererPixbuf()                                                                                                      
        object_column = gtk.TreeViewColumn('Object')        
        object_column.pack_start(textrenderer, expand=False)
        object_column.set_attributes(textrenderer, cell_background=2,
                                     foreground=3, text=0)
        textrenderer2 = gtk.CellRendererText()
        size_column = gtk.TreeViewColumn('Numbers')
        size_column.pack_start(textrenderer2, expand=False)        
        size_column.set_attributes(textrenderer2, cell_background=2,
                                     foreground=3, text=1)
                                                       
        self.append_column(object_column)
        self.append_column(size_column)
        
        self._object_col = object_column

        # right click context menu
        self.connect("button-release-event", self.slot_button_released)             

    def selection_changed_handler(self, selection):            
        model, iter = selection.get_selected()
        if iter:
            dim, selection = model.get_value(iter,0) , model.get_value(iter,4)
            logger.information.inform_about_selected(dim,selection)
                                                                                                                                                                                                                                
    def slot_button_released(self, list, event):
        self.selection_changed_handler(self.get_selection())
        
        x, y = map(int, event.get_coords())
        path = self.get_path_at_pos(x, y)
        if not path:
            return
        
        if event.button==3:        
            # right button pressed! Show Context Menu!
            widget_tree  = gtk.glade.XML(self._gladefilename,
                                     'selections_context_menu')
            invert = widget_tree.get_widget("invert1")
            invert.connect("activate",self.invert_selection_at_path,path[0])
            remove = widget_tree.get_widget("remove1")            
            remove.connect("activate",self.remove_selection_at_path,path[0])
            store = widget_tree.get_widget("store_selection1")
            store.connect("activate",self.store_selection_at_path,path[0])            
                                                 
            # connect stuff!
            popupmenu = widget_tree.get_widget('selections_context_menu')
            popupmenu.popup(None, None, None, 0, 0)                                                  

    def invert_selection_at_path(self,ignored, path):
        model = self.get_model()
        iter = model.get_iter(path)
        data_best = 0
        if iter:
            dim = model.get_value(iter,0)
            sel = model.get_value(iter,4)
            datas = self._project.get_current_data()
            target = None
            for data in datas:
                if dim in data.get_dim_names():
                    target = data
                    break
            if not target: # get a one with the same dimension
                hits = 0
                #get the root node
                tree = self._project.data_tree.get_iter_root() # rootnode
                #if self._project.data_tree.iter_has_child(treeiter) :
                while tree:
                    treeiter = self._project.data_tree.iter_children(tree)
                    while treeiter :
                        dataset = self._project.data_tree.get_object_at_iter(treeiter)
                        dim_names = self._project.data_tree.get_object_at_iter(treeiter).get_dim_names()
                       
                        if dim in dim_names:
                            if len(set(sel).intersection(dataset.get_identifiers(dim))) > hits :
                                hits = len(set(sel).intersection(dataset.get_identifiers(dim)))
                                data_best = dataset
                        treeiter = self._project.data_tree.iter_next(treeiter)
                    tree = self._project.data_tree.iter_next(tree)
            if not target :
                data = data_best
                if not data :
                    logger.log("error", "No valid dataset can be found.")
                    return
            ids = list(set(data.get_identifiers(dim)).difference(sel))
            ids.sort()  # sorted order is better than random (from set operations)
            model.set_selection(dim,ids)  
            self._project.notify_selection_listeners()                                                                          
        else:
            logger.log("error","Attempted to invert selection which doesn't exist. Action aborted, as succesfully inverting the empty set would make the universe explode.")    

    def remove_selection_at_path(self, ignored, path):        
        model = self.get_model()
        iter = model.get_iter(path)
        if iter:
            dim = model.get_value(iter,0)
            model.set_selection(dim,[])                                                                            
        else:
            logger.log("error","Attempted to remove non-existing selection. Action aborted, as large amounts of anti-matter may be potentially harmfull to health.")                             

    def store_selection_at_path(self, ignored, path):
        model = self.get_model()
        iter = model.get_iter(path)
        if iter:
            dim = model.get_value(iter,0)
            sel = model.get_value(iter,4)

            self._project.add_selection_data( Selection(model,dim,sel))
        else:
            logger.log("error","Attempted to store non-existing element. This would not have been very constructive")


class StandardOperations:

    def autoscale(self, data):
        ids = [(dim, data.get_identifiers(dim)) for dim in data.get_dim_names()]
        newdata = dataset.Dataset(data.asarray().copy() , name = data._name, identifiers = ids)
        x = newdata.shape
        templist = [] # stores index elements were variance is zero
        tempvalues = []
        for i in x:
            for k in range(x[1]): #2 dimensional
                if scipy.std(newdata.asarray()[:,k]) == 0 :
                    #save positions and replace first element with a number
                    tempvalues.append(newdata.asarray()[0,k])
                    newdata.asarray()[0,k] = 5
                    templist.append(k)
    
        rpy.r.assign("x",newdata.asarray())
        rpy.r("x = scale(x)")
        dims = [(dim,newdata.get_identifiers(dim)) for dim in newdata.get_dim_names()]
        newdata = dataset.Dataset(rpy.r('x'),dims,name="Autoscaled " + newdata.get_name())       
        center = rpy.r('attr(x,"scaled:center")')
        if type(center) != type([]) : center = [center]
        centered = map(lambda x : str(x)[0:5] , center)  #list(rpy.r('attr(x,"scaled:center")')))   
        scale = rpy.r('attr(x,"scaled:scale")')
        if type(scale) != type([]) : scale = [scale]
        scaled = map(lambda x : str(x)[0:5] , scale) # list(rpy.r('attr(x,"scaled:scale")')))
        # resets rows where the standard ddeviation is zero
        for i in range(len(templist)):
            centered[i] = str(tempvalues[i])
            scaled[i] = str("NaN")
            for k in range(len(newdata.asarray()[0:templist[i]])) : # if 2 dimensional
                # if the std for any row is zero, it will be reset to zero for all values
                newdata.asarray()[k:templist[i]] = newdata.asarray()[k:templist[i]] + ( newdata.asarray()[k:templist[i]]*-1)
        newdata.add_to_description("<b>Centered with:</b>:" + " :: ".join(centered))
        newdata.add_to_description("<b>Scaled by:</b>:" + " :: ".join(scaled))       
        return [newdata]

    def transpose(self, data):
        new_array = Numeric.transpose(data.asarray())
        dim_ids = [(dim_, data.get_identifiers(dim_)) for dim_ in data.get_dim_names()]
        dim_ids.reverse()
        ndata = [dataset.Dataset(new_array, dim_ids,name="T("+data.get_name()+")")]
        ndata[0].add_to_description("<b>Transposed from: </b>" + data.get_name())
        return ndata

    def center(self, data):
        rpy.r.assign("x",data.asarray())
        rpy.r("x = scale(x,scale=FALSE)")
        dims = [(dim,data.get_identifiers(dim)) for dim in data.get_dim_names()]
        data = dataset.Dataset(rpy.r('x'),dims,name="Centered " + data.get_name())
        centered = map(lambda x : str(x)[0:5] , list(rpy.r('attr(x,"scaled:center")')))       
        data.add_to_description("<b>Centered with:</b>:" + " :: ".join(centered))
        return [data]

    def msc(self, data):
        if not ensure_r_libraries(["pls"]):
            return None
        rpy.r.assign("data",data.asarray())
        res = rpy.r("msc(data)")
        dims = [(dim,data.get_identifiers(dim)) for dim in data.get_dim_names()]
        return [dataset.Dataset(res,dims,name="msc: "+data.get_name())]            
                
        
    def subset(self, data):
        dialog = gtk.Dialog("Subset",buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        table = gtk.Table()
        dialog.vbox.pack_start(table)
        widgets = []

        table.attach(gtk.Label("Dimension Name"),0,1,0,1)
        table.attach(gtk.Label("From:"),1,2,0,1)
        table.attach(gtk.Label("To:"),2,3,0,1)

        # create widgets, add text
        for dim in data.get_dim_names():
            widgets.append([])
            widgets[-1].append(gtk.Label(dim))
            widgets[-1].append(gtk.combo_box_new_text())
            for num,id in enumerate(data.get_identifiers(dim)):
                widgets[-1][-1].append_text(str(num + 1) + " : " + id)
            widgets[-1][-1].set_active(0)
            widgets[-1].append(gtk.combo_box_new_text())
            widgets[-1][-1].set_model(widgets[-1][-2].get_model())
            widgets[-1][-1].set_active(len(data.get_identifiers(dim))-1)
            
        for y,widget in enumerate(widgets):
            for x,w in enumerate(widget):
                table.attach(w,x,x+1,y+1,y+2)
        table.show_all()
        
        try:
            if dialog.run()!=gtk.RESPONSE_ACCEPT:
                return []
        finally:
            dialog.destroy()

        # now grab actual subset
        dims = []
        array = data.asarray()
        for x,(dim,widget) in enumerate(zip(data.get_dim_names(),widgets)):
            order = [widget[1].get_active(), widget[2].get_active()]
            order.sort()
            order[1] += 1               # both ends included!
            indices = range(*order)
            ids = data.get_identifiers(dim)[order[0]:order[1]]
            dims.append((dim,ids))
            array = Numeric.take(array,indices,x)
        data = dataset.Dataset(array,dims,name="Subset: " + data.get_name()) 

        return [data]

    def reduce_dim(self,data) :
        def data_func(data):
            pass

        min_dim = data.get_dimension_length(data.get_dim_names()[0])
        min_dim_name = data.get_dim_names()[0]
        information = ""
        for name in data.get_dim_names() :
            if  data.get_dimension_length(name) < min_dim :
                min_dim = data.get_dimension_length(name)
                min_dim_name = name
            information += name + " (" + str(data.get_dimension_length(name)) + ")" + "\n"
        

        dialog = gtk.Dialog(title="Reduce dimension", buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        hbox = gtk.HBox()
        infoframe = gtk.Frame(label="Dataset" )
        selframe = gtk.Frame(label="Reduce dimension")
        label = gtk.Label("Dimension:")
        combo = gtk.Combo()
	names = data.get_dim_names()
	names.insert(0, names.pop(names.index(min_dim_name)))
        combo.set_popdown_strings(names)
        combo.entry.connect("activate", data_func, data)
        infoframe.set_border_width(5)
        selframe.set_border_width(5)
        
        table = gtk.Table()
        
        table.attach(gtk.Label(str(information)),0,1,0,1)
        
        hbox.pack_start(label)
        hbox.pack_start(combo)
        infoframe.add(table)
        selframe.add(hbox)
        dialog.vbox.pack_start(infoframe)
        dialog.vbox.pack_start(selframe)            
        dialog.show_all()

        try:
            if dialog.run() != gtk.RESPONSE_ACCEPT :
                return
        finally:
            dialog.destroy()
        new_datasets = []
        selected = combo.entry.get_text()
        arr = data.asarray().copy()
        element = data.get_dim_names().index(selected)
        n = []
        for name in data.get_dim_names() :
            if name != selected :
                n.append(data.get_dimension_length(name))
        ids = [(dim, data.get_identifiers(dim))
                for dim in data.get_dim_names()]

        ids.pop(element)
        for num in range(arr.shape[element]) :            
            newdata = dataset.Dataset(Numeric.reshape( Numeric.take(arr, [num], axis=element), (n) ), \
                name= str(selected) + str(num), identifiers = ids)   
            new_datasets.append(newdata)

        return new_datasets

# this should really, really be moved.        
def ensure_r_libraries(libs):
    """Ensure packages in list libs are installed locally for user.

    Returns True on successful install or already installed.  Will
    show GUI to user to confirm install if not already installed.
    """
    # check if we have a working rpy
    try:
        import rpy
        rpy.r
    except Exception, e:
        logger.log("error", "RPy not properly installed.")
        return False
     
    lib_paths = rpy.with_mode(rpy.VECTOR_CONVERSION, rpy.r._libPaths)()
    if os.name=="posix":
        path = os.path.join(os.getenv("HOME"), ".scicraft-ng", "rlibs")
        rpy.r._libPaths([path] + lib_paths)
    else:
        # we assume directory is writable
        pass
    
    install_packages = []
    # do we have libraries?
    for lib in libs:
        try:
            rpy.r.library(lib)
            # yes
        except: 
            # no
            install_packages.append(lib)

    if install_packages == []:
        # all libraries are installed
        return True

    # for unixes we make sure path to install in is writable
    if os.name == "posix":
        lib_paths = rpy.with_mode(rpy.VECTOR_CONVERSION, rpy.r._libPaths)()
        path = lib_paths[0]
        if not os.access(path, os.R_OK | os.W_OK | os.X_OK):
            if os.name == "posix":                      
                path = os.path.join(os.getenv("HOME"), ".scicraft-ng", "rlibs")
            else:
                path = os.path.join(os.getenv("USERPROFILE"), "SciCraft-NG",
                                    "rlibs")
            if not os.path.exists(path):
                os.mkdir(path)
            rpy.r._libPaths([path] + lib_paths)
    
    message = ""
    for i in install_packages:    
        message += "'" + i + "', "
    message = message[:-2]
    install_path = rpy.with_mode(rpy.VECTOR_CONVERSION, rpy.r._libPaths)()[0]
    if not dialogs.get_yes_no("Installing R packages", """\
                                The following package(s) are missing: %s
                                Do you wish to install into '%s'?

                                If you do not install the package(s), the function will not be run.
                                """  % (message, install_path)):
        logger.log("warning",
                   "Not installing missing packages, aborting function.")
        return False
                              
    logger.log("notice", "Installing package(s): %s into '%s'" % (message,
                                                                  install_path))
    logger.log("notice", "Please be patient while installation is running.")
    silent_eval = rpy.with_mode(rpy.NO_CONVERSION, rpy.r)

    while(gtk.events_pending()):
        gtk.main_iteration()
    from system.config import r_CRAN_mirror 
    for package in install_packages:
        silent_eval("install.packages('%s',repos='%s', lib='%s')" % \
                    (package, r_CRAN_mirror,install_path))
    return True
        
        
                                                                                            
