import os
import sys
import scipy
import gobject
import gtk
import time
from system import dataset, plots, dialogs, logger

# used by load_icon()
_cached_icons = {}
_iconpath = os.path.join(os.path.dirname(sys.modules['system'].__file__),
                         "../icons")

def load_icon(filename):
    """Load filename from iconpath.
    
    If icon has been previously loaded, a cached version is
    returned. This implies that changing an icon on disk after
    SciCraft has started will have no effect.
    """
    if filename not in _cached_icons:
        path = os.path.join(_iconpath, filename)
        _cached_icons[filename] = gtk.gdk.pixbuf_new_from_file(path)
    return _cached_icons[filename]


class CategoryItem:
    def __init__(self, title):
        self._title = title

    def get_name(self):
        return self._title

    # for tests
    def __eq__(self, other):
        try:
            return self._title == other._title
        except AttributeError:
            return False

    def __ne__(self, other):
        return not self == other


class ItemStore(gtk.TreeStore):
    def __init__(self):
        gtk.TreeStore.__init__(self, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT,
                               gobject.TYPE_STRING, gobject.TYPE_STRING,
                               gobject.TYPE_BOOLEAN, gobject.TYPE_OBJECT)
        self.use_function_icon = True

    def set_use_function_icon(self,bool):
        self.use_function_icon = bool                               

    def get_object_at_path(self, path):
        iter = self.get_iter(path)
        return self.get_object_at_iter(iter)
        
    def get_object_at_iter(self, iter):
        return self.get_value(iter, 1)

    def import_data_tree(self, root, iter=None):
        """Import tree model from node tree."""
        
        for child in root.children:
            value = child.value
            child_path = self.get_path(self._insert(iter, value))
            self.import_data_tree(child, child_path)        

    def extract_data_tree(self):
        """Return values in tree model as node tree.

        Creates a tree structure that is picklable (storable to file).
        """
        def parse_tree(parent, current_iter):
            # Recursively create nodes from children of current_iter
            # as children of parent.
            for i in range(self.iter_n_children(current_iter)):
                child = self.iter_nth_child(current_iter, i)
                
                node = TreeNode()
                data = self.get_object_at_iter(child)
                node.value = self.get_object_at_iter(child)
                parent.children.append(node)

                parse_tree(node, child)
        
        root = TreeNode()
        current_iter = None

        parse_tree(root, current_iter)
        return root

    def _insert(self, parent, data):
        """Create and insert stuff into treestore."""
        bgcolour = "white"
        text = data.get_name()

        if isinstance(data, dataset.Dataset):
            fontcolour = "blue"
            if len(data.shape)<3:
                icon = load_icon("data.png")
            else:
                icon = load_icon("data3d.png")
        elif isinstance(data, plots.Plot):
            fontcolour = "dark green"
            icon = load_icon("plot.png")
                
        elif isinstance(data, Notice):
            fontcolour = "dark red"            
            icon = load_icon("text.png")
        elif isinstance(data, Selection):
            fontcolour = "red"
            icon = None            
        else:
            fontcolour = "black"
            if self.use_function_icon:
                icon = load_icon("function.png")
            else:
                icon = None
        if parent:                    
            it = self.append(self.get_iter(parent))
        else:
            it = self.append(None)
        self.set_value(it, 0, text)         # text appearing on menu
        self.set_value(it, 1, data)         # Actual object
        self.set_value(it, 2, bgcolour)     # Background colour
        self.set_value(it, 3, fontcolour)   # Font colour   
        self.set_value(it, 4, False)     # Boolean for 'editable' value
        self.set_value(it, 5, icon)         #pixbuf containing icon
        
        return it

    def _find_object(self, obj):
        """Find the iter pointing at an object."""
        iters = [self.get_iter_first()]
        while(iters and self.get_value(iters[0],1) != obj):
            iter = iters.pop(0)
            if self.iter_children(iter):
                iters += [self.iter_children(iter)]
            if self.iter_next(iter):
                iters += [self.iter_next(iter)]
        if iters:    
            return iters[0]
        return None

    def _get_object_from_text(self, text):
        """Find iter from text"""
        
        iters = [self.get_iter_first()]
        while(iters and self.get_value(iters[0],0) != text):
            iter = iters.pop(0)
            if self.iter_children(iter):
                iters += [self.iter_children(iter)]
            if self.iter_next(iter):
                iters += [self.iter_next(iter)]
        if iters:    
            return [self.get_value(iters[0],1)]
        return None            
        
    def add_data(self, parents, data, fun='Function', place_at_bottom=False):    
        """Add a set of data and plots to the navigator.

        This method is usually called after a Function in a workflow
        has finished and returns its output."""

        if place_at_bottom:
            parents=None

        iter = None                
        path = None
        # determine where data is inserted
        if parents and not place_at_bottom:
            iter = self._find_object(parents[0])
            if iter:
                path = self.get_path(iter)
                
        if fun or not iter:
            # category 
            it = self._insert(path, CategoryItem(fun))
        else:
            it = iter
        
        # add substuff to category
        for d in data:
            self._insert(self.get_path(it), d)

    def get_all_plots(self):
        all_objects = self.get_objects_from_path("0")
        return [obj for obj in all_objects if isinstance(obj, plots.Plot)]

    def get_objects_from_path(self, path):
        """Get all objects _below_ (descendents of) path.

        Categories (function names) will not be returned.
        """
        def collector(all_objects, current_iter):
            for i in range(self.iter_n_children(current_iter)):
                child = self.iter_nth_child(current_iter, i)
                data = self.get_object_at_iter(child)
                all_objects.append(data)
                collector(all_objects, child)
                
        all_objects = []
        iter = self.get_iter(path)
        collector(all_objects, iter)
        return filter(None, all_objects)
        
    def set_renamable_by_path(self, path, value=True):
        iter = self.get_iter(path)
        self.set_value(iter, 4, value)

    def set_text_by_path(self, path, text):
        iter = self.get_iter(path)
        # change displayed value
        self.set_value(iter, 0 , text)
        # also change internal value
        obj = self.get_object_at_iter(iter)
        if isinstance(obj, dataset.Dataset):
            obj.set_name(text)
        elif isinstance(obj, plots.Plot):
            obj.set_title(text)

    def delete_items(self, paths):
        """Deletes one or more items from the datatree.

        plot_view is needed to remove plots from plot area when
        they've been deleted.
        """
        for path in paths:
            try:
                iter = self.get_iter(path)
            except ValueError:
                # ancestor was already deleted
                continue
            parent = self.iter_parent(iter)
            # delete item
            self.remove(iter)
            # ... and if parent is a label with only one child, delete that too
            if parent and not self.iter_has_child(parent):
                parent_obj = self.get_object_at_iter(parent)
                if not isinstance(parent_obj, dataset.Dataset):
                    self.remove(iter)

    def path_has_children(self, path):
        """Check if path has children."""
        return self.iter_has_child(self.get_iter(path))             

class SelectionStore(gtk.TreeStore):
    def __init__(self,project):
        gtk.TreeStore.__init__(self,gobject.TYPE_STRING, gobject.TYPE_STRING,
                               gobject.TYPE_STRING, gobject.TYPE_STRING,gobject.TYPE_PYOBJECT)
        self._project = project
        self._selections = {}                               
    
    def set_selection(self,dim,selection):
        logger.information.inform_about_selected(dim,selection)
        self._selections[dim] = selection
        self.update_selection()
        self._project.notify_selection_listeners()
        
        # this would have been conseptually better with signals, but this is easier
        self._project.notify_selection_listeners()
        
    def update_selection(self):
        self.clear()
        for dim, selection in self._selections.items():
            if selection:
                self.append(None,(dim,str(len(selection)),"white","blue",selection))

class Project:
    def __init__(self, filename=None):
        self.data_tree = ItemStore()
        self.selection_tree = SelectionStore(self)
        self._selection_observers = []
        self._current_data = []
        self._current_selection = {}
        self._filename = filename
        self._dirty = False

    def connect_selection(self, target):
        """Set up target as both listener and source of selections."""
        target.set_selection_listener(self.set_selection)
        self._selection_observers.append(target)
        
        # notify target of selections
        target.selection_changed(self.get_selection())
        

    def set_filename(self, new_name):
        """Mark file as saved under new filename."""
        self._filename = new_name
        self._dirty = False

    def get_filename(self):
        return self._filename

    def is_dirty(self):
        return self._dirty

    def import_data_tree(self, root):
        """Import tree model from node tree.

        Will not mark project as dirty as it implies data exists on file.
        """
        self.data_tree.import_data_tree(root, None)
        for plot in self.data_tree.get_all_plots():
            self.connect_selection(plot)
            
    def notify_selection_listeners(self):
        for observer in self._selection_observers:
            observer.selection_changed(self.get_selection())
        
    def set_selection(self, dim_name, selection):
        """Set current selection and notifies observers."""
        self.selection_tree.set_selection(dim_name, selection)

        
    def get_selection(self):
        """Return current selection."""
        return self.selection_tree._selections

    def extract_data_tree(self):
        """Return values in tree model as node tree.

        Creates a tree structure that is picklable (storable to file).
        """
        return self.data_tree.extract_data_tree()
 
    def add_data(self, parents, data, fun='Function', place_at_bottom=False):
        """Add a set of data and plots to the navigator.

        This method is usually called after a Function in a workflow
        has finished and returns its output."""
        self.data_tree.add_data(parents, data, fun, place_at_bottom)
        for d in data:
            if isinstance(d, plots.Plot):
                self.connect_selection(d)
        self._dirty = True

    def add_selection_data(self, selection):
        """ Add a Selection to the navigagor"""
        cat  = "Selection: " + selection._dim
        obj = self.data_tree._get_object_from_text(cat)
        if obj:
            self.add_data(obj,[selection],fun=None)
        else:
            self.add_data(None,[selection],fun=cat)                                
        
    def set_current_data(self, obj):
        """Set currently selected datasets."""
        self._current_data = obj

    def get_current_data(self):
        """Get currently selected datasets."""
        return self._current_data


class TreeNode:
    def __init__(self):
        self.children = []
        self.value = None

    def __repr__(self):
        return "<TreeNode value=%r children=%r>" % (self.value, self.children)
    
    def __eq__(self, other):
        return self.children == other.children and \
               self.value == other.value

    def __ne__(self, other):
        return not self == other


# these should be probably be moved somewhere; I can't decide where though
class ActivatableItem:
    def __init__(self, name):
        self.name = name

    def get_name(self):
        return self.name
        
    def set_name(self, name):
        self.name = name
                                
    def activate(self):
        pass

class Notice(ActivatableItem):
    """Shows notices/logs on the navigatormenu."""
    def __init__(self, name, text):
        ActivatableItem.__init__(self,name)
        self._text = str(text)
    
    def activate(self):
        logger.information.set_markup(self._text)

class Selection(ActivatableItem):
    def __init__(self, model, dim, selection):
        ActivatableItem.__init__(self,str(len(selection))+" on " + dim)
        
        self._model = model
        self._dim  = dim
        self._selection = selection

    def activate(self):
        self._model.set_selection(self._dim, self._selection)                
