PCA <- function(X,permutations,Ncomp){

#Performes PCA on a data matrix, also has the option of
# computing the null distribution of the eigenvalues.

# INPUT VARIABLES
# X - matrix of data
# permutations - number of randomly permuted matrices used to
#                compute the null distribution of the eigenvalues. 
#
# OUTPUT VRIABLES
# pcaModel      - an R-object containing the following variables
#  
#              sdev: the standard deviations of the principal components (i.e.,
#              the square roots of the eigenvalues of the
#              covariance/correlation matrix, though the calculation is
#              actually done with the singular values of the data matrix).
#
#              rotation: the matrix of variable loadings
#                  (i.e., a matrix whose columns
#                  contain the eigenvectors).  The function 'princomp' returns
#                  this in the element 'loadings'.
#
#              x: if 'retx' is true the value of the rotated and centred
#                 data multiplied by the 'rotation'
#                      matrixis returned.
#
#              center,: the centering used.
#
# ComponentPvalues -  vector containing the estimated null
# distribution of the eigenvalues
  
# Copyright(C), Endre Andersen and Jon Strom, The SciCraft Team, www.scicraft.org
#Last update: 14/3-06
# $$$ ## This file is part of SciCraft.
# $$$ ##
# $$$ ## SciCraft is free software; you can redistribute it and/or modify it
# $$$ ## under the terms of the GNU General Public License as published by
# $$$ ## the Free Software Foundation; either version 2, or (at your option)
# $$$ ## any later version.
# $$$ ##
# $$$ ## SciCraft is distributed in the hope that it will be useful, but
# $$$ ## WITHOUT ANY WARRANTY; without even the implied warranty of
# $$$ ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# $$$ ## General Public License for more details.
# $$$ ##
# $$$ ## You should have received a copy of the GNU General Public License
# $$$ ## along with R; see the file COPYING.  If not, write to the Free
# $$$ ## Software Foundation, 59 Temple Place - Suite 330, Boston, MA
# $$$ ## 02111-1307, USA.
# $$$ 
# $$$ ## -*- texinfo -*-
#
#
#Compute PCA model
  if(Ncomp==0) {
    warning("number of components not specified, set to ncol(X). ")
    Ncomp <- ncol(X)
  }
  dX <- dim(X)
  X <- as.matrix(X)
  if(is.null(dX)) {
    warning("wrong input. X should be a matrix")
  } 
  pcaModel <- prcomp(X,scale.=T)
  pValues <- 0
  
  #Compute null distribution of eigenvalues (lambdas)
  if (permutations > 0) {
    lambdas <- matrix(0,permutations,min(dX)) 
    for (i in 1:permutations){
      Xp <- X
      for (j in 1:dX[2]){  
        Xp[,j] <- sample(X[,j]) 
      }       
      pcaModelP <- prcomp(Xp,scale.=T)
      lambdas[i,] <- pcaModelP$sdev^2
    } 
    
    pValues <- vector("numeric",length=length(pcaModelP$sdev))   
    for (k in 1:length(pcaModel$sdev)){
      pValues[k] <- (sum(as.vector(lambdas) >= pcaModel$sdev[k]))/(permutations*min(dim(X)))  
    }   
  }
  if(Ncomp==0) {
    warning("number of components not specified, reset. ")
    Ncomp <- ncol(pcaModel$rotation)
  }
  if(Ncomp>ncol(pcaModel$rotation)) {
    warning("Selected more components than the dimension of the
      matrix. Reset. ")
    Ncomp <- ncol(pcaModel$rotation)
  }
  L <- pcaModel$rotation
  for(i in 1:Ncomp) {
    L[,i] <- L[,i]*pcaModel$sdev[i]
  }
  residMatrix <- round(cor(X)-L%*%t(L),getOption("digits"))
  lambda <- pcaModel$sdev^2
  names(pcaModel) <-
      c("Eigenvalues","LoadingsMatrix","center","scale","ScoresMatrix")
  pcaModel$Eigenvalues <- lambda
  return(list(pcaModel=pcaModel,ComponentPvalues=pValues,ResidualMatrix=residMatrix))
}
#Compute Full PCA model



#Resample to determine optimum number of components
#Cross validate to determine loading stability
#Compute loading T^2 scores
#Compute leverage
#Compute sample and variable residuals
#Compute X residuals
