import gtk , gtk.glade
import gobject

import os.path
import cPickle
import sys
import webbrowser
from StringIO import StringIO

from system import dataset, logger, plots, workflow, dialogs, project
from system.plots import ScatterPlot, ScatterPlotToolbar, CustomPopupMenu


from scipy import randn, array, transpose, zeros
from matplotlib.figure import Figure
from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
import Numeric, scipy


def ensure_bio_conductor():
    """Ensure BioConductor is installed locally for user.

    Returns True on successful install or already installed.  Will
    show GUI to user to confirm install if not already installed.
    """
    # do we have a working rpy?
    try:
        import rpy
        rpy.r
    except Exception, e:
        logger.log("error", "RPy not properly installed.")
        return False
    
    # do we have affy?
    try:
        rpy.r.library("affy")
        # yes
        return True
    except: pass

    # for unixes we make sure path to install in is writable
    if os.name == "posix":
        lib_paths = rpy.with_mode(rpy.VECTOR_CONVERSION, rpy.r._libPaths)()
        path = lib_paths[0]
        if not os.access(path, os.R_OK | os.W_OK | os.X_OK):
            if os.name == "posix":
                path = os.path.join(os.getenv("HOME"), ".scicraft-ng", "rlibs")
            else:
                path = os.path.join(os.getenv("USERPROFILE"), "SciCraft-NG", "rlibs")

            if not os.path.exists(path):
                os.mkdir(path)
            rpy.r._libPaths([path] + lib_paths)


    install_path = rpy.with_mode(rpy.VECTOR_CONVERSION, rpy.r._libPaths)()[0]
    if not dialogs.get_yes_no("Installing BioConductor", """\
The BioConductor-library is not installed.
Do you wish to install it into '%s'?

If you do not install it, the function will not be run.
""" % install_path):
        logger.log("warning", "Not installing BioConductor, aborting function.")
        return False
                              
    logger.log("notice", "BioConductor not installed, installing into '%s'" % install_path)
    logger.log("notice", "Please be patient while installation is running.")
    while(gtk.events_pending()):
        gtk.main_iteration()
    rpy.r.source("http://www.bioconductor.org/biocLite.R")
    rpy.r.biocLite()
    rpy.r.rm(rpy.r.ls())
    return True


class AffyWorkflow (workflow.Workflow):

    name = 'Affy Workflow'
    ident = 'affy'
    description = 'Affymetrics Workflow. Analysis of Affy-data.'
    def __init__(self, app):
        workflow.Workflow.__init__(self, app)

        load = workflow.Stage('load', 'Load Data')
        load.add_function(CelFileImportFunction())
        load.add_function(PhenotypeImportFunction())
        self.add_stage(load)

        significance = workflow.Stage('significance', 'Significance analysis')
        significance.add_function(LimmaFunction())
        self.add_stage(significance)

        explore = workflow.Stage('explore', 'Explorative analysis')
        explore.add_function(PCAFunction(self))
        explore.add_function(SimplePCAFunction(self))
        explore.add_function(PrintFunction())
        self.add_stage(explore)

        plot = workflow.Stage('plot', 'Plot data')
        plot.add_function(ScatterPlotFunction())
        plot.add_function(DensityFunction('density','Density Plot'))
        self.add_stage(plot)

        help = workflow.Stage('help', 'Help')
        help.add_function(ShowExampleFunction())
        self.add_stage(help)


class ShowExampleFunction(workflow.Function):
    def __init__(self):
        workflow.Function.__init__(self, 'show_example', 'Show Example')

    def run(self):
        webbrowser.open_new('https://dev.scicraft.org/trac/wiki/AffymetricsWorkflowExample')


class PrintFunction(workflow.Function):
    def __init__(self):
        workflow.Function.__init__(self, 'printer', 'Print Stuff')

    def run(self, data):
        dim1, dim2 = data.get_dim_names()
        for row in zip(data.get_identifiers(dim1), data.asarray().tolist()):
            print "\t".join(map(str, row))        

class DatasetSaveFunction(workflow.Function):
    """QND way to save data to file for later import to this program."""
    def __init__(self):
        workflow.Function.__init__(self, 'save_data', 'Save Pickled Dataset')

    def run(self, data):
        filename = dialogs.get_save_filename("Save pickled data...",
                                             data.get_name() + ".pkl",
                                             ("Python pickled data files", "pkl"))
        if filename:
            cPickle.dump(data, open(filename, "wb"), protocol=2)
            logger.log("notice", "Saved data to %r." % filename)


class LimmaFunction(workflow.Function):
    def __init__(self):
        workflow.Function.__init__(self, 'limma', 'Limma')

    def run(self, affydata, phenotypes):
        
        #Verify that the imput data is given in the correct order (affydata, phenotypes)
        if not isinstance(phenotypes, PhenotypeDataset):
            logger.log("error", "The second input to the Limma function must be phenotypes data")
    
        response = dialogs.get_text('Enter contrasts...', """\
Enter comma-separated list of contrasts.
Available categories: %s

Example: Y-N, M-F""" % ", ".join(phenotypes.get_categories()))

        logger.log("notice", "contrasts selected: %s" % response)

        categories = []
        [categories.extend(s.split("-")) for s in response.split(",")]
        categories = [s.strip() for s in categories]

        bad_categories = set(categories) - set(phenotypes.get_categories())
        if bad_categories:
            logger.log("error", "aborted function. invalid categories %s." % ", ".join(bad_categories))
            return

        factors = phenotypes.get_factors(categories)
        if not factors:
            logger.log("warning", "nothing to do, no factors")

        table = phenotypes.get_phenotype_table([os.path.splitext(f)[0] for f in affydata.get_identifiers('filename')])
        cn = table[0]
        entries = zip(*table[1:])
        rn = entries[0]

        import rpy
        silent_eval = rpy.with_mode(rpy.NO_CONVERSION, rpy.r)

        if not ensure_bio_conductor():
            return None
        
        rpy.r.library("limma")
        silent_eval("a <- matrix('kalle', nrow=%d, ncol=%d)" % (len(rn), len(cn)))
        for i, row in enumerate(entries):
            for j, entry in enumerate(row):
                silent_eval("a[%d, %d] <- '%s'" % (j+1, i+1, entry))
        rpy.r.assign("rn", rn)
        rpy.r.assign("cn", cn)
        silent_eval("rownames(a) <- rn")
        silent_eval("colnames(a) <- cn")

        unique_categories = list(set(categories))

        # compose fancy list of factors for design matrix
        silent_eval("design <- matrix(0, nrow=%d, ncol=%d)" % (len(rn), len(unique_categories)))
        for i, category in enumerate(unique_categories):
            for j, value in enumerate(phenotypes.get_category_variable(category)):
                silent_eval("design[%d, %d] <- %d" % (j+1, i+1, value))

        rpy.r.assign("colnames.design", unique_categories)
        silent_eval("colnames(design) <- colnames.design")

        rpy.r.assign("expr", affydata.asarray())
        silent_eval("fit <- lmFit(expr, design)")
        silent_eval("contrast.matrix <- makeContrasts(%s, levels=design)" % response)
        silent_eval("fit2 <- contrasts.fit(fit, contrast.matrix)")
        silent_eval("fit2 <- eBayes(fit2)")
        coeff = rpy.r("fit2$coefficients")
        amean = rpy.r("fit2$Amean")
        padj = rpy.r("p.adjust(fit2$p.value, method='fdr')")

        dim_1, dim_2 = affydata.get_dim_names()


        coeff_data = dataset.Dataset(coeff, [(dim_1, affydata.get_identifiers(dim_1)),
                                             ("contrast", [response])],
                                     name="Coefficients")
        amean_data = dataset.Dataset(array(amean), [("average", ["average"]),
                                                    (dim_1, affydata.get_identifiers(dim_1))],
                                     name="Average Intensity")
        padj_data = dataset.Dataset(padj, [(dim_1, affydata.get_identifiers(dim_1)),
                                           ("contrast", [response])],
                                    name="Adjusted P-value")

        vulcano_plot = ScatterPlot(coeff_data, padj_data,('contrast',response), 
                                          ('contrast', response),name="Vulcano plot",use_objectnames=False)

        # We should be nice and clean up after ourselves
        rpy.r("rm(list=ls())")

        return [coeff_data, amean_data, padj_data, vulcano_plot]
        

class CelFileImportFunction(workflow.Function):
    """Loads Affymetrics .CEL-files into matrix."""
    def __init__(self):
        workflow.Function.__init__(self, 'cel_import', 'Import Affy')

    def run(self):
        import rpy

        filenames = dialogs.get_open_filename("Select cel files...",
                                              ("Cel Files", "cel"),
                                              multiple=True)
        if not filenames:
            return
        if len(filenames) == 1:
            logger.log("warning", "Only single Affymetrics file selected, nothing to compare.")
            return

        if not ensure_bio_conductor():
            return None

        while gtk.events_pending():
            gtk.main_iteration()
        
        rpy.r.library("affy")

        silent_eval = rpy.with_mode(rpy.NO_CONVERSION, rpy.r)

        # make sure we can write to the first entry of .libPaths()
        # if not:
        #    add $HOME/.scicraft/rlibs to .libPaths()

        if os.name == "posix":
            lib_paths = rpy.with_mode(rpy.VECTOR_CONVERSION, rpy.r._libPaths)()
            path = lib_paths[0]
            if not os.access(path, os.R_OK | os.W_OK | os.X_OK):
                path = os.path.join(os.getenv("HOME"), ".scicraft-ng", "rlibs")
                if not os.path.exists(path):
                    os.mkdir(path)
            rpy.r._libPaths([path] + lib_paths)

        if os.name == 'nt':
            filenames = [file_.replace("\\", "/") for file_ in filenames]


        silent_eval('E <- ReadAffy(filenames=c("%s"))' % '", "'.join(filenames))
        silent_eval('E <- rma(E)')
        m = rpy.r('m <- E@exprs')

        vector_eval = rpy.with_mode(rpy.VECTOR_CONVERSION, rpy.r)
        rownames = vector_eval('rownames(m)') 
        colnames = vector_eval('colnames(m)') 

        # We should be nice and clean up after ourselves
        rpy.r.rm(["E", "m"])

        if m:
            data = dataset.Dataset(m, (('ids', rownames), ('filename', colnames)), name="Affymetrics Data")
            plot = plots.LinePlot(data, "Gene profiles")
            return [data, plot]
        else:
            logger.log("notice", "No data loaded from importer.")


class PhenotypeImportFunction(workflow.Function):
    def __init__(self):
        workflow.Function.__init__(self, 'import_phenotype', 'Import Phenotypes')

    def run(self):
        filename = dialogs.get_open_filename("Load phenotypes description...")
        if filename:
            text = open(filename).read()
            data = PhenotypeDataset(text)
            return [data]

class SimplePCAFunction(workflow.Function):
    """<b>Principal Component Analysis</b>: Performs a PCA analysis, does not compute leverage
    
Accepts:
Any matrix

Outputs
Scores, Loadings
Loadingplot, Scoreplot, Biplot"""
    def __init__(self, wf):
        workflow.Function.__init__(self, 'pcanoextra', 'PCA (Simple Plots)')
        self._workflow = wf

    def run(self, data):
        import rpy
        
        X_dim_1, X_dim_2 = data.get_dim_names()
        X_ids_1, X_ids_2 = data.get_identifiers(X_dim_1) , data.get_identifiers(X_dim_2)
        
        if (len(data.get_identifiers(X_dim_1)) < 2):
            logger.log("error", "This data set is not suitable for performing a PCA analysis. Number of variables less than 2")
            return
        silent_eval = rpy.with_mode(rpy.NO_CONVERSION, rpy.r)
        rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)("m", data.asarray())
        
        # scale the variable to unit variance as default.

        silent_eval("t = prcomp(m, center.=TRUE)")

        pca_dim_name = dataset.make_dim_name_unique("PCA")
        
        p = dataset.Dataset(rpy.r("t$rotation"), [(X_dim_2, X_ids_2),
                                                  (pca_dim_name, None)], name="P")                                                
        pca_ids = p.get_identifiers(pca_dim_name)
        
        t = dataset.Dataset(rpy.r("t$x"),[(X_dim_1, X_ids_1),
                                           (pca_dim_name, pca_ids)], name="T") 

        from workflows.chemometrics_workflow import ScatterPlotBrowse
        from workflows.chemometrics_workflow import Biplot
        
        returnlist = [t,p]
        returnlist.append(ScatterPlotBrowse(p, p,  
                                        (p[1],p[1,0]),
                                        (p[1],p[1,1]), 
                                        name = "Loadings", 
                                        yaxis_label = "Principal Component: %s ",
                                        xaxis_label = "Principal Component: %s ",
                                        use_objectnames=False))
        returnlist.append(ScatterPlotBrowse(t, t,  
                                        (t[1],t[1,0]),
                                        (t[1],t[1,1]),
                                        name = "Scores",
                                        yaxis_label = "Principal Component: %s ", 
                                        xaxis_label = "Principal Component: %s ",
                                        use_objectnames=False))
                                        
        returnlist.append(Biplot(t,p,p[1],t[1],t[1,0],t[1,1],use_objectnames=False))                                           
        return returnlist                                                   

class PCAFunction(workflow.Function):
    """<b>Principal Component Analysis</b>: Performs a PCA analysis.
    
Accepts:
Any matrix

Outputs
Scores, Loadings, Centering Matrix
Loadingplot, Scoreplot, Biplot, Influence Plot, Residual Variance Plot, Leverage Plot"""
    def __init__(self, wf):
        workflow.Function.__init__(self, 'pca', 'PCA')
        self._workflow = wf

    def run(self, data):
        import rpy
        
        X_dim_1, X_dim_2 = data.get_dim_names()
        X_ids_1, X_ids_2 = data.get_identifiers(X_dim_1) , data.get_identifiers(X_dim_2)
        
        if (len(data.get_identifiers(X_dim_1)) < 2):
            logger.log("error", "This data set is not suitable for performing a PCA analysis. Number of variables less than 2")
            return
        silent_eval = rpy.with_mode(rpy.NO_CONVERSION, rpy.r)
        rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)("m", data.asarray())
        
        # scale the variable to unit variance as default.

        silent_eval("t = prcomp(m, center.=TRUE)")

        pca_dim_name = dataset.make_dim_name_unique("PCA")
        
        p = dataset.Dataset(rpy.r("t$rotation"), [(X_dim_2, X_ids_2),
                                                  (pca_dim_name, None)], name="P")                                                
        pca_ids = p.get_identifiers(pca_dim_name)
        
        t = dataset.Dataset(rpy.r("t$x"),[(X_dim_1, X_ids_1),
                                           (pca_dim_name, pca_ids)], name="T")

        center = dataset.Dataset(rpy.r("t$center"),[(X_dim_2,X_ids_2)],name="Centering Matrix")                                           
                                           
        lambda_values = rpy.r("t$sdev^2")
        # This must be chosen from the ExplainedVariancePlot, for now we're using a defaulta value.
        # Make GUI for number of eigenvalues to be used in the further analysis:
        
        # THIS VALUE SHOULD BE GIVEN FROM USER
        MAGIC_CONSTANT = 0.75
        
        a = int(rpy.r("dim(t$rotation)[2]")*0.75)    
           
        t_new = t.asarray()[:,0:a]
        p_new = p.asarray()[:,0:a]
        
        t_new_dataset = dataset.Dataset(t_new, [(X_dim_1, X_ids_1),
                                                (pca_dim_name, pca_ids[0:a])]) 
        p_new_dataset = dataset.Dataset(p_new, [(X_dim_2, X_ids_2),
                                                (pca_dim_name, pca_ids[0:a])])
                                                
        rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)("T", t_new)
        rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)("P", p_new)
        
        # leverage of X-variabeles:
        leverage_variable = rpy.r("as.matrix(diag(P%*%t(P)))")
                
        leverage_variable_dataset = dataset.Dataset(leverage_variable, 
                                                    [(X_dim_2, X_ids_2), ("Leverage", None)]) 
       
        residuals_variables = rpy.r("matrix(apply(sqrt((m-as.matrix(T%*%t(P)))^2),2,sum))")
        res_data_dataset = dataset.Dataset(residuals_variables, 
                                           [(X_dim_2, X_ids_2), ("Residuals", None)])
                                           
        explained_variance = []
        sum_lambda_values = sum(lambda_values)
        for i in range(1, len(lambda_values) + 1):
            explained_variance.append([float(sum(lambda_values[0:i])) * 100 / sum_lambda_values])
                     
        explained_variance = dataset.Dataset(explained_variance,[(pca_dim_name,pca_ids),(None,None)])
        explained_variance._array = 100 - explained_variance._array  
            
        # cleanup
        rpy.r.rm(["t", "m", "T", "P"])
        returnlist = [t,p, center]

        from workflows.chemometrics_workflow import ExplainedVariancePlot
        from workflows.chemometrics_workflow import LeveragePlot
        from workflows.chemometrics_workflow import ScatterPlotBrowse
        from workflows.chemometrics_workflow import Biplot
        from workflows.chemometrics_workflow import InfluencePlot
        
        returnlist.append(ScatterPlotBrowse(p, p,  
                                        (p[1],p[1,0]),
                                        (p[1],p[1,1]), 
                                        name = "Loadings", 
                                        yaxis_label = "Principal Component: %s ",
                                        xaxis_label = "Principal Component: %s "))
        returnlist.append(ScatterPlotBrowse(t, t,  
                                        (t[1],t[1,0]),
                                        (t[1],t[1,1]),
                                        name = "Scores",
                                        yaxis_label = "Principal Component: %s ", 
                                        xaxis_label = "Principal Component: %s "))
                                        
        returnlist.append(Biplot(t,p,p[1],t[1],t[1,0],t[1,1]))
        returnlist.append(InfluencePlot(t,p,data,center))
                                        
                                            
                                            
        dim = explained_variance.get_dim_names()[1]
        id  = explained_variance.get_identifiers(dim)[0]                                            
        returnlist.append(ExplainedVariancePlot(explained_variance,dim,id,
                                            xaxis_label="Principal Component"))
        
        returnlist.append(LeveragePlot(t,p,xaxis_label="Sample", yaxis_label="Leverage"))
                                 
        
        return returnlist 


class PhenotypeDataset(dataset.Dataset):
    def __init__(self, string):
        self._table = rows = [line.split("\t") for line in string.splitlines()]
        columns = zip(*rows[1:])
        cel_names = columns[0]
        col_names = rows[0][1:]
        phenotypes = []
        categories = {}
        self._categories = {}

        for col_name, column in zip(col_names, columns[1:]):
            try:
                categories[col_name] = map(float, column)
                phenotypes.append(col_name)
            except ValueError:
                # category-data
                keys = []
                entries = {}
                for i, entry in enumerate(column):
                    if entry not in entries:
                        keys.append(entry)
                        entries[entry] = []

                    entries[entry].append(i)
                    
                for key in keys:
                    self._categories[key] = col_name
                    z = zeros(len(column))
                    for i in entries[key]:
                        z[i] = 1
                    key = "%s-%s" % (col_name, key)
                    phenotypes.append(key)
                    categories[key] = z

        matrix_data = []
        for key in phenotypes:
            matrix_data.append(categories[key])

        if matrix_data:
            a = transpose(array(matrix_data))
        else:
            a = ([],) * len(cel_names)

        dataset.Dataset.__init__(self, a, identifiers=[('CEL', cel_names),
                                                       ('phenotypes', phenotypes)],
                                 name="Phenotype Data")

    def sort_cels(self, cel_names):
        self._dims = []

        cels = {}
        for row in self._table[1:]:
            cels[row[0]] = row[1:]

        new_table = [self._table[0]]
        for name in cel_names:
            new_table.append([name] + cels[name])

        self._table = new_table
        self._set_identifiers([('CEL', cel_names), ('phenotypes', self.get_identifiers('phenotypes'))])
            
    def get_phenotype_table(self, cel_order=None):
        """Get string based table of phenotypes as read from file."""
        if not cel_order:
            return self._table
        else:
            cels = {}
            for row in self._table[1:]:
                cels[row[0]] = row[1:]

            new_table = [self._table[0]]
            for name in cel_order:
                new_table.append([name] + cels[name])

            return new_table

    def get_categories(self):
        """Get categories of factors.

        If factor 'sick' had possibilites Y/N, and 'sex' M/F, the
        categories would be Y, N, M and F.
        """
        return self._categories.keys()

    def get_factors(self, categories):
        factors = set()
        for c in categories:
            factors.add(self._categories[c])

        return factors

    def get_category_variable(self, category):
        # abit brute-force, but does the job until optimization is
        # necessary
        factor = self._categories[category]
        variable = []
        for column in zip(*self.get_phenotype_table()):
            if column[0] == factor:
                for entry in column[1:]:
                    if entry == category:
                        variable.append(1)
                    else:
                        variable.append(0)

        return variable


class ScatterPlotFunction(workflow.Function):
    def __init__(self):
        workflow.Function.__init__(self, 'scatter_function', 'Scatterplot')

    def run(self, data_1, data_2=None):
        if not data_2:
            data_2 = data_1

        xml = gtk.glade.XML(os.path.join(os.path.split(__file__)[0],
                                         "scatterdialog.glade"), 'dialog')
        window = xml.get_widget('dialog')
        x_combo = xml.get_widget('x_combo')
        y_combo = xml.get_widget('y_combo')

        def fill_combo(combo, data):
            dim_1, dim_2 = data.get_dim_names()
            # The shortest dimension is used for selecting what to plot
            if len(data.get_identifiers(dim_1)) < len(data.get_identifiers(dim_2)):
                dim = dim_1
                other_dim = dim_2
            else:
                dim = dim_2
                other_dim = dim_1

            store = gtk.ListStore(gobject.TYPE_STRING)
            for ident in data.get_identifiers(dim):
                store.insert_before(None, (ident,))

            cell = gtk.CellRendererText()
            combo.set_model(store)
            combo.set_text_column(0)
            combo.child.set_editable(False)
            combo.set_active(0)

            return dim, other_dim

        x_dim, x_other_dim = fill_combo(x_combo, data_1)
        y_dim, y_other_dim = fill_combo(y_combo, data_2)

        x_length = len(data_1.get_identifiers(x_other_dim))
        y_length = len(data_2.get_identifiers(y_other_dim))
        if x_length != y_length:
            logger.log("warning", "Can't plot datasets of different sizes (%d versus %d)." % (x_length, y_length))

        res = window.run()
        try:
            if res == gtk.RESPONSE_OK:
                return [plots.ScatterPlot(data_1, data_2, x_other_dim, x_dim, y_dim,
                                          x_combo.get_active_text(),
                                          y_combo.get_active_text())]
        finally:
            window.destroy()

class DensityFunction(workflow.Function):
    def run(self,data):
        return [DensityPlot(data)]


class DensityPlot(plots.Plot):
    def __init__(self, data):
        plots.Plot.__init__(self,"Density Plot")
        import rpy
        self.lines = []
        self.colours = []
        self._data = data
        self._dim = data.get_dim_names()[list(data.shape).index(min(data.shape))]
        self._ids = data.get_identifiers(self._dim)
        for id in self._ids:
            rpy.r.assign("x",data.get_data_slice(self._dim,id))
            self.lines.append(rpy.r("density(x)$y"))
            self.colours.append(scipy.rand(3))
        
         

        fig = Figure(figsize=(5,4), dpi=72)
        self._canvas = FigureCanvas(fig)
        self._ax = fig.add_subplot(111)        
        self.add(self._canvas)
        self._canvas.show()
        
        self._init_toolbar()
        self._init_popupmenu()                                         
        self.update()

    def get_popupmenu(self):
        return self._popupmenu
            
    def get_toolbar(self):            
        return self._toolbar
   
    def _init_popupmenu(self):
        self._popupmenu = CustomPopupMenu(self)
        self._popupmenu.init_from_toolbar(self.get_toolbar())                                

    def _init_toolbar(self):
        # create toolbar
        self._toolbar = ScatterPlotToolbar(self._canvas, None) 
        self._toolbar.set_property('show-arrow', False)                                                
                
    def update(self, selections=None):
        self._ax.clear()
        self._ax.set_title(self.get_name())
        drawn_lines = []         
        for colour, line in zip(self.colours, self.lines) :        
            drawn_lines += self._ax.plot(range(len(line)),line,color=colour)
        
        if selections:
            # get indices from ids
            indices = self._data.get_indices(self._dim,selections)
            for num in indices:
                line = self.lines[num]
                self._ax.plot(range(len(line)),line,"r_")
        self._canvas.draw()
                    
    def selection_changed(self,selection):
        if self._dim in selection:
            # we have an selection, get union of selected and available
            ids = [id for id in self._ids if id in selection[self._dim]]                        
            
            self.update(selections = ids)
                                    
            
                
        
          


class ExportTableFunction(workflow.Function):
    def __init__(self):
        workflow.Function.__init__(self, 'export_table', 'Export Dataset(s)')

    def run(self, selection, data, *rest):
        text = self.convert((data,) + rest, selection)
        filename = dialogs.get_save_filename("Save tab-separated data...",
                                             data.get_name() + ".txt")
        if filename:
            open(filename, "w").write(text)
            logger.log("notice", "Saved data to %r." % filename)

    def convert(self, datalist, selection=None):
        if len(datalist) == 1:
            dim_1, dim_2 = datalist[0].get_dim_names()
            if len(datalist[0].get_identifiers(dim_1)) > len(datalist[0].get_identifiers(dim_2)):
                dim = dim_1
            else:
                dim = dim_2
        else:

            dims = set(datalist[0].get_dim_names())
            for d in list(dims):
                if len(datalist[0].get_identifiers(d)) == 1:
                    dims.remove(d)
                    
            for data in datalist[1:]:
                dims.intersection_update(data.get_dim_names())

            dim, = dims

        buf = StringIO()

        rownames = datalist[0].get_identifiers(dim)

        column_names = [dim]
        columns = []
        for data in datalist:
            names = data.get_dim_names()
            names.remove(dim)
            dim_name, = names
            if 1 in data.asarray().shape:
                column_names.append(data.get_name())
            elif len(datalist) == 1:
                    column_names.extend(data.get_identifiers(dim_name))
            else:
                if 1 in datalist[0].asarray().shape:
                    column_names.append(datalist[0].get_name())
                column_names.extend(["%s_%s" % (dim_name, ident) for ident in data.get_identifiers(dim_name)])

            if dim == data.get_dim_names()[1]:
                columns.extend(data.asarray().tolist())
            else:
                columns.extend(zip(*data.asarray().tolist()))
            
        print >> buf, "\t".join(column_names)

        print_all = not selection or dim not in selection or len(selection[dim]) == 0

        for rowname, row in zip(rownames, zip(*columns)):
            if print_all or rowname in selection[dim]:
                print >> buf, "\t".join([rowname] + map(str, row))

        return buf.getvalue()
        
